#include "CivHydrTableHelper.h"
#include "CivTableFields.h"
#include "CivCommonUtils.h"
#include "CivPgDbConnection.h"

CivHydrTableHelper::CivHydrTableHelper(const std::string& uri)
{
	mConn = new CivPgDbConnection();
	mConn->connect(uri);
}

CivHydrTableHelper::~CivHydrTableHelper()
{
	mConn->disconnect();
	delete mConn;
	mConn == nullptr;
}

bool CivHydrTableHelper::mainTain()
{
	unTopoMaintain();

	// 
	updateAssem(TABLE_NODE, std::string(""), "JD");
	updateAssem(TABLE_PIPE, std::string(""), "GD");
	updateAssem(TABLE_TANK, std::string(""), "TK");
	updateAssem(TABLE_VALVE, std::string(""), "VA");
	updateAssem(TABLE_RESERVOIR, std::string(""), "RE");
	updateAssem(TABLE_PUMP, std::string(""), "PU");
	
	// 
	updateAssem(PROJ_TABLE_NODE, std::string(""), "PJD");
	updateAssem(PROJ_TABLE_PIPE, std::string(""), "PGD");
	updateAssem(PROJ_TABLE_TANK, std::string(""), "PTK");
	updateAssem(PROJ_TABLE_VALVE, std::string(""), "PVA");
	updateAssem(PROJ_TABLE_RESERVOIR, std::string(""), "PRE");
	updateAssem(PROJ_TABLE_PUMP, std::string(""), "PPU");

	if (!topoMaintain())
		return false;

	return true;
}

void CivHydrTableHelper::updateAssem(const std::string& table, const std::string& filed,const std::string& prefix)
{
	// ڵ
	std::vector<std::map<std::string, std::string>> resVec;
	if (!mConn->query(table, { "id" }, resVec))
	{
		mErrInfo = mConn->getLastError();
		return;
	}
	
	if (resVec.size() <= 0)
		return;

	std::vector<std::string> sNs;
	CivCommonUtils::genSeqNo(prefix, resVec.size(), sNs);

	std::string sql = "update "+ table +" set "+ filed +" = tmp."+ filed +" from(values";
	for (int i = 0; i < sNs.size(); i++)
	{
		std::map<std::string,std::string> resMap = resVec[i];
		std::string id = resMap.find("id")->second;
		sql.append("(");
		sql.append(id);
		sql.append(",\'");
		sql.append(sNs[i]);
		sql.append("\'),");
	}
	sql = sql.substr(0, sql.length() - 1);
	sql += ") as tmp(id, \""+ filed +"\") where "+ table +".id = tmp.id; ";

	if (mConn && !mConn->execSql(sql))
	{
		mErrInfo = mConn->getLastError();
		return;
	}
}

bool CivHydrTableHelper::topoMaintain()
{
	// ܶ
	topoMainTable(TABLE_PIPE, TABLE_NODE, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(TABLE_PIPE, TABLE_NODE, std::string("ֹڵ"), std::string(""), std::string("to_code"));
	topoMainTable(TABLE_PIPE, TABLE_RESERVOIR, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(TABLE_PIPE, TABLE_TANK, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(TABLE_PIPE, TABLE_VALVE, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(TABLE_PIPE, TABLE_VALVE, std::string("ֹڵ"), std::string(""), std::string("to_code"));

	topoMainTable(TABLE_PIPE, PROJ_TABLE_NODE, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(TABLE_PIPE, PROJ_TABLE_NODE, std::string("ֹڵ"), std::string(""), std::string("to_code"));
	topoMainTable(TABLE_PIPE, PROJ_TABLE_RESERVOIR, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(TABLE_PIPE, PROJ_TABLE_TANK, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(TABLE_PIPE, PROJ_TABLE_VALVE, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(TABLE_PIPE, PROJ_TABLE_VALVE, std::string("ֹڵ"), std::string(""), std::string("to_code"));

	topoMainTable(PROJ_TABLE_PIPE, TABLE_NODE, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(PROJ_TABLE_PIPE, TABLE_NODE, std::string("ֹڵ"), std::string(""), std::string("to_code"));
	topoMainTable(PROJ_TABLE_PIPE, TABLE_NODE, std::string("ʼڵ"), std::string(""), std::string("to_code"));
	topoMainTable(PROJ_TABLE_PIPE, TABLE_NODE, std::string("ֹڵ"), std::string(""), std::string("from_code"));
	topoMainTable(PROJ_TABLE_PIPE, TABLE_RESERVOIR, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(PROJ_TABLE_PIPE, TABLE_TANK, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(PROJ_TABLE_PIPE, TABLE_VALVE, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(PROJ_TABLE_PIPE, TABLE_VALVE, std::string("ֹڵ"), std::string(""), std::string("to_code"));

	topoMainTable(PROJ_TABLE_PIPE, PROJ_TABLE_NODE, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(PROJ_TABLE_PIPE, PROJ_TABLE_NODE, std::string("ֹڵ"), std::string(""), std::string("to_code"));
	topoMainTable(PROJ_TABLE_PIPE, PROJ_TABLE_RESERVOIR, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(PROJ_TABLE_PIPE, PROJ_TABLE_TANK, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(PROJ_TABLE_PIPE, PROJ_TABLE_VALVE, std::string("ʼڵ"), std::string(""), std::string("from_code"));
	topoMainTable(PROJ_TABLE_PIPE, PROJ_TABLE_VALVE, std::string("ֹڵ"), std::string(""), std::string("to_code"));

	// ͷŵ˹ϵ
	topoValve(TABLE_PIPE, TABLE_VALVE,"ֹڵ", "from_code", "to_code");
	topoValve(TABLE_PIPE, TABLE_VALVE, "ʼڵ", "to_code", "from_code");

	topoValve(TABLE_PIPE, TABLE_VALVE, "ʼڵ", "to_code", "from_code", TABLE_RESERVOIR);
	topoValve(TABLE_PIPE, TABLE_VALVE, "ʼڵ", "to_code", "from_code",TABLE_TANK);
	
	// ܶ
	topoValve(PROJ_TABLE_PIPE, TABLE_VALVE, "ֹڵ", "from_code", "to_code");
	topoValve(PROJ_TABLE_PIPE, TABLE_VALVE, "ʼڵ", "to_code", "from_code");

	topoValve(PROJ_TABLE_PIPE, PROJ_TABLE_VALVE, "ֹڵ", "from_code", "to_code");
	topoValve(PROJ_TABLE_PIPE, PROJ_TABLE_VALVE, "ʼڵ", "to_code", "from_code");

	topoValve(TABLE_PIPE, PROJ_TABLE_VALVE, "ֹڵ", "from_code", "to_code");
	topoValve(TABLE_PIPE, PROJ_TABLE_VALVE, "ʼڵ", "to_code", "from_code");

	topoValve(TABLE_PIPE, TABLE_VALVE, "ʼڵ", "to_code", "from_code", TABLE_RESERVOIR);
	topoValve(TABLE_PIPE, TABLE_VALVE, "ʼڵ", "to_code", "from_code", TABLE_TANK);

	topoValve(PROJ_TABLE_PIPE, PROJ_TABLE_VALVE, "ʼڵ", "to_code", "from_code", PROJ_TABLE_RESERVOIR);
	topoValve(PROJ_TABLE_PIPE, PROJ_TABLE_VALVE, "ʼڵ", "to_code", "from_code", PROJ_TABLE_TANK);

	// ˮά
	topoPump(TABLE_PIPE, TABLE_PUMP, "ʼڵ", "from_code", "to_code", TABLE_TANK);
	// topoPump(TABLE_PIPE, TABLE_PUMP, "ʼڵ", "from_code", "to_code", TABLE_RESERVOIR);
	topoPump(TABLE_PIPE, TABLE_PUMP, "ֹڵ", "to_code", "from_code", TABLE_NODE);
	//topoPump(TABLE_PIPE, TABLE_PUMP, "ʼڵ", "from_code", "to_code", TABLE_TANK);

	//topoPump(TABLE_PIPE, TABLE_PUMP, "ʼڵ", "from_code", "to_code", TABLE_NODE);
	//topoPump(TABLE_PIPE, TABLE_PUMP, "ֹڵ", "to_code", "from_code", TABLE_NODE);

	//topoPump(PROJ_TABLE_PIPE, PROJ_TABLE_PUMP, "ʼڵ", "from_code", "to_code", TABLE_NODE);
	//topoPump(PROJ_TABLE_PIPE, PROJ_TABLE_PUMP, "ֹڵ", "to_code", "from_code", TABLE_NODE);

	//topoPump(PROJ_TABLE_PIPE, PROJ_TABLE_PUMP, "ʼڵ", "from_code", "to_code", PROJ_TABLE_RESERVOIR);
	//topoPump(PROJ_TABLE_PIPE, PROJ_TABLE_PUMP, "ʼڵ", "from_code", "to_code", PROJ_TABLE_TANK);
	return true;
}

bool CivHydrTableHelper::unTopoMaintain()
{
	unTopoTable(TABLE_NODE, {std::string("")});
	unTopoTable(TABLE_PIPE, { std::string(""),std::string("ʼڵ"),std::string("ֹڵ") });
	unTopoTable(TABLE_RESERVOIR, { std::string("")});
	unTopoTable(TABLE_TANK, { std::string("") });
	unTopoTable(TABLE_VALVE, { std::string(""),std::string("ʼڵ"),std::string("ֹڵ") });
	unTopoTable(TABLE_PUMP, { "","ʼڵ","ֹڵ"});

	unTopoTable(PROJ_TABLE_NODE, { std::string("") });
	unTopoTable(PROJ_TABLE_PIPE, { std::string(""),std::string("ʼڵ"),std::string("ֹڵ") });
	unTopoTable(PROJ_TABLE_RESERVOIR, { std::string("") });
	unTopoTable(PROJ_TABLE_TANK, { std::string("") });
	unTopoTable(PROJ_TABLE_VALVE, { std::string(""),std::string("ʼڵ"),std::string("ֹڵ") });
	unTopoTable(PROJ_TABLE_PUMP, { "","ʼڵ","ֹڵ" });
	return true;
}

bool CivHydrTableHelper::topoMainTable(const std::string& sourceTable, 
	const std::string& targetTable, 
	const std::string& pipesn, 
	const std::string& nodeSn,
	const std::string& code)
{
	std::string sql = "UPDATE \"" + sourceTable + "\" \
		SET \"" + pipesn + "\" = cqt.\"" + nodeSn + "\" FROM \"" + targetTable + "\" cqt WHERE \
		 \""+sourceTable+"\".\"" + code + "\" = cqt.\"code\"";

	if (mConn && !mConn->execSql(sql))
	{
		mErrInfo = mConn->getLastError();
		return false;
	}

	return true;
}

bool CivHydrTableHelper::topoLikelyPipe(const std::string& pipeTable, const std::string& nodeTable, const std::string& filed, const std::string& tocode)
{
	std::string sql = "update \""+nodeTable+"\" as pu set \""+filed+"\" = \
		(select pipe.\"\" from \""+pipeTable+"\" as pipe  where pipe.\""+tocode+"\"=pu.\"code\")  ";

	if (mConn && !mConn->execSql(sql))
	{
		mErrInfo = mConn->getLastError();
		return false;
	}

	return true;
}

bool CivHydrTableHelper::topoValve(const std::string& pipeTable,
	const std::string& valveTable,
	const std::string& sn, 
	const std::string& fromcode, 
	const std::string& tocode,
	const std::string& nodeTable)
{
	std::string sql = "update \"";
	sql.append(valveTable);
	sql.append("\" set \"" + sn + "\" = dd.\"\"\
		from(select node.\"\" \"\", node.\"code\" \"nodecode\", tt.\"code\"  \"vacode\" from \
			\""+ nodeTable +"\" node, (select t2.\""+tocode+"\" \""+tocode+"\", t1.\"code\" \"code\" from \""+ pipeTable +"\" t2, \""+ valveTable +"\" t1 where t2.\""+fromcode+"\" = t1.\"code\") tt\
			where node.\"code\" = tt.\""+tocode+"\") dd\
		where \"code\" = dd.\"vacode\"");


	if (mConn && !mConn->execSql(sql))
	{
		mErrInfo = mConn->getLastError();
		return false;
	}



	return true;
}

bool CivHydrTableHelper::unTopoTable(const std::string& table, const std::vector<std::string>& fields)
{
	if (fields.size() < 1)
	{
		mErrInfo = "fileds is empty";
		return false;
	}
		
	std::string sql = "update \""+ table +"\" set ";
	for (int i = 0; i < fields.size(); i++)
	{
		sql.append(fields[i]);
		sql.append("= null");
		sql.append(",");
	}
	sql = sql.substr(0, sql.length() - 1);

	
	if (mConn && !mConn->execSql(sql))
	{
		mErrInfo = mConn->getLastError();
		return false;
	}
	return false;
}

void CivHydrTableHelper::getCodeToSn(const std::string& code, std::string& sn)
{
	std::string sql = "select \"\" from  ڵ where code = '" + code+"'";

	if (mConn && !mConn->execSql(sql))
	{
		mErrInfo = mConn->getLastError();
		return;
	}

	std::vector<std::map<std::string, std::string>> res;
	mConn->queryResult(res);

	if (res.size() <= 0)
	{
		return;
	}

	sn = res[0].find("")->second;
	return;
}

bool CivHydrTableHelper::topoPump(
	const std::string& pipeTable,
	const std::string& pumpTable,
	const std::string& sn,
	const std::string& fromcode,
	const std::string& tocode,
	const std::string& nodeTable)
{
	std::string psql = "update \""+ pumpTable +"\" set \""+sn+"\" = gg.\"\" from (select node.\"\",bb.\"vcode\" from  \"ˮ\" as node,\
		(select pipe.\""+fromcode+"\" as fromcode, a.\"code\" vcode from \""+pipeTable+"\" as pipe inner join \""+ pumpTable +"\" as  a on pipe.\""+tocode+"\" = a.\"code\")bb\
	where node.\"code\" = bb.\"fromcode\") gg\
		where \"code\" = gg.\"vcode\" ";


	std::string pumpSQL = "update \"ˮ\" set \"ʼڵ\" = gg.\"\" from (select node.\"\",bb.\"vcode\" from  \"ˮ\" as node,\
		(select pipe.\"from_code\" as fromcode, a.\"code\" vcode from \"ܶ\" as pipe inner join \"ˮ\" as  a on pipe.\"to_code\" = a.\"code\")bb\
	where node.\"code\" = bb.\"fromcode\") gg\
		where \"code\" = gg.\"vcode\" ";

	if (mConn && !mConn->execSql(pumpSQL))
	{
		mErrInfo = mConn->getLastError(); 
		return false;
	}

	std::string sedpumpSQL = "update \"ˮ\" set \"ֹڵ\" = gg.\"\" from (select node.\"\",bb.\"vcode\" from  \"ڵ\" as node,\
		(select pipe.\"to_code\" as tocode, a.\"code\" vcode from \"ܶ\" as pipe inner join \"ˮ\" as  a on pipe.\"from_code\" = a.\"code\")bb\
	where node.\"code\" = bb.\"tocode\") gg\
		where \"code\" = gg.\"vcode\"; ";

	if (mConn && !mConn->execSql(sedpumpSQL))
	{
		mErrInfo = mConn->getLastError();
		return false;
	}

	return true;
}

std::string CivHydrTableHelper::splicingPumpSql(const std::string& pumpTable, const std::string& nodetable)
{
	std::string sql = "";
	return sql;
}
